﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using Excel = Microsoft.Office.Interop.Excel;
using Office = Microsoft.Office.Core;
using Microsoft.Office.Tools.Excel;
using System.IO;
using System.Windows.Forms;

namespace CCSProjectExcel2013AddIn
{
    public partial class ThisAddIn
    {

        private Excel.Application _app;
        private List<string> _unicodeFiles;
        private bool _flag = false;

        //Unicode file byte order marks.
        private const string UTF_16BE_BOM = "FEFF";
        private const string UTF_16LE_BOM = "FFFE";
        private const string UTF_8_BOM = "EFBBBF";

        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            _app = this.Application;
            _unicodeFiles = new List<string>();
            _app.WorkbookOpen += new Excel.AppEvents_WorkbookOpenEventHandler(WorkbookOpenHandler);
            _app.WorkbookBeforeClose += new Excel.AppEvents_WorkbookBeforeCloseEventHandler(WorkbookBeforeCloseHandler);
            _app.WorkbookBeforeSave += new Excel.AppEvents_WorkbookBeforeSaveEventHandler(WorkbookBeforeSaveHandler);
        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
            _app = null;
            _unicodeFiles = null;
        }

        void WorkbookBeforeSaveHandler(Excel.Workbook Wb, bool SaveAsUI, ref bool Cancel)
        {
            if (!SaveAsUI && !_flag && _unicodeFiles.Contains(Wb.FullName))
            {
                Cancel = true;
                SaveAsUnicodeCSV(false, false);
            }
            _flag = false;
        }

        protected override Office.IRibbonExtensibility CreateRibbonExtensibilityObject()
        {
            return new SaveAsUnicodeRibbon();
        }

        void WorkbookBeforeCloseHandler(Excel.Workbook Wb, ref bool Cancel)
        {
            _unicodeFiles.Remove(Wb.FullName);
            _app.StatusBar = "Ready";
        }

        void WorkbookOpenHandler(Excel.Workbook Wb)
        {
            if (Wb.FullName.ToLower().EndsWith(".csv") && isFileUnicode(Wb.FullName))
            {
                if (!_unicodeFiles.Contains(Wb.FullName))
                {
                    _unicodeFiles.Add(Wb.FullName);
                }
                _app.StatusBar = Wb.Name + " has been opened as a Unicode CSV file";
            }
            else
            {
                _app.StatusBar = "Ready";
            }
        }

        private bool isInCellEditingMode()
        {
            bool flag = false;
            try
            {
                _app.DisplayAlerts = false; 
            }
            catch (Exception)
            {
                flag = true;
            }
            return flag;
        }

        public void SaveAsUnicodeCSV(bool force, bool newFile)
        {
            _app.StatusBar = "";
            bool currentDisplayAlert = _app.DisplayAlerts;
            bool flag = true;
            int index;
            string filename = _app.ActiveWorkbook.FullName;

            if (force)
            {
                if (newFile || !filename.ToLower().EndsWith(".csv"))
                {
                    Office.FileDialog d = _app.get_FileDialog(Office.MsoFileDialogType.msoFileDialogSaveAs);
                    index = _app.ActiveWorkbook.Name.LastIndexOf(".");
                    if (index >= 0)
                    {
                        d.InitialFileName = _app.ActiveWorkbook.Name.Substring(0, index);
                    }
                    else
                    {
                        d.InitialFileName = _app.ActiveWorkbook.Name;
                    }
                    d.AllowMultiSelect = false;
                    Office.FileDialogFilters f = d.Filters;
                    for (index = 1; index <= f.Count; index++)
                    {
                        if ("*.csv".Equals(f.Item(index).Extensions))
                        {
                            d.FilterIndex = index;
                            break;
                        }
                    }
                    if (d.Show() == 0) //User cancelled the dialog
                    {
                        flag = false;
                    }
                    else
                    {
                        filename = d.SelectedItems.Item(1);
                    }
                }
                if (flag && !filename.ToLower().EndsWith(".csv"))
                {
                    MessageBox.Show("Please select a CSV file name first");
                    flag = false;
                }
            }

            if (flag && filename.ToLower().EndsWith(".csv") && (force || _unicodeFiles.Contains(filename)))
            {
                if (isInCellEditingMode())
                {
                    MessageBox.Show("Please finish editing before saving");
                }
                else
                {
                    try
                    {
                        //Getting current selection to restore the current cell selection
                        Excel.Range rng = (Excel.Range)_app.ActiveCell;
                        int row = rng.Row;
                        int col = rng.Column;

                        string tempFile = System.IO.Path.GetTempFileName();

                        try
                        {
                            _flag = true; 
                            _app.ActiveWorkbook.SaveAs(tempFile, Excel.XlFileFormat.xlUnicodeText);
                            _app.ActiveWorkbook.Close();

                            if (new FileInfo(tempFile).Length <= (1024 * 1024)) 
                            {
                                File.WriteAllText(filename, File.ReadAllText(tempFile, UnicodeEncoding.UTF8).Replace("\t", ","), UnicodeEncoding.UTF8);
                            }
                            else 
                            {
                                using (StreamReader sr = new StreamReader(tempFile, true))
                                using (StreamWriter sw = new StreamWriter(filename, false, sr.CurrentEncoding))
                                {
                                    char[] buffer = new char[10 * 1024]; //10KB Chunks
                                    while (!sr.EndOfStream)
                                    {
                                        int cnt = sr.ReadBlock(buffer, 0, buffer.Length);
                                        for (index = 0; index < cnt; index++)
                                        {
                                            if (buffer[index] == '\t')
                                            {
                                                buffer[index] = ',';
                                            }
                                        }
                                        sw.Write(buffer, 0, cnt);
                                    }
                                }
                            }
                        }
                        finally
                        {
                            File.Delete(tempFile);
                        }

                        _app.Workbooks.Open(filename, Type.Missing, Type.Missing, Excel.XlFileFormat.xlCSV, Type.Missing, Type.Missing, Type.Missing, Type.Missing, ",");
                        Excel.Worksheet ws = _app.ActiveWorkbook.ActiveSheet;
                        ws.Cells[row, col].Select();
                        _app.StatusBar = "File has been saved as a Unicode CSV";
                        if (!_unicodeFiles.Contains(filename))
                        {
                            _unicodeFiles.Add(filename);
                        }
                        _app.ActiveWorkbook.Saved = true;
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show("Error occured while trying to save this file as Unicode CSV: " + e.Message);
                    }
                    finally
                    {
                        _app.DisplayAlerts = currentDisplayAlert;
                    }
                }
            }
        }

        private bool isFileUnicode(string filename)
        {
            bool ret = false;
            try
            {
                byte[] buff = new byte[3];
                using (FileStream fs = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.ReadWrite))
                {
                    fs.Read(buff, 0, 3);
                }

                string hx = "";
                foreach (byte letter in buff)
                {
                    hx += string.Format("{0:X2}", Convert.ToInt32(letter));
                    //Checking to see the first bytes matches with any of the defined Unicode BOM
                    //We only check for UTF8 and UTF16 here.
                    ret = UTF_16BE_BOM.Equals(hx) || UTF_16LE_BOM.Equals(hx) || UTF_8_BOM.Equals(hx);
                    if (ret)
                    {
                        break;
                    }
                }
            }
            catch (IOException)
            {
                //ignore any exception
            }
            return ret;
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }
        
        #endregion
    }
}
