﻿using System;
using System.Collections.Generic;
using System.Reflection;

namespace CCSPrototypeCommon
{
    public static partial class Utility
    {
        public class LatLngModel
        {
            public Double? lat;
            public Double? lng;
            public LatLngModel(Double? lat, Double? lng)
            {
                this.lat = lat;
                this.lng = lng;
            }
        }

        public static int ConvertExcelColumnToNum(string column)
        {
            if (string.IsNullOrWhiteSpace(column))
            {
                return -1;
            }
            column = column.ToUpper();
            int result = 0;
            int sequence = 0;
            char[] charArray = column.ToCharArray();
            for (var index = charArray.Length - 1; index >= 0; --index )
            {
                int columnNum = charArray[index] - 'A' + 1;
                if (sequence == 0)
                {
                    result = columnNum;
                    sequence = 26;
                }
                else
                {
                    result += sequence * columnNum;
                    sequence *= 26;
                }
            }
            return result - 1;
        }

        public static string GetCustomerIDFromBlobName(string blobName)
        {
            // Blob path format: CustomerID/yyyy/MMdd/HHmmss.fileExtension
            int index = blobName.IndexOf('/');

            return (index < 0) ? blobName : blobName.Remove(index);
        }

        public static List<string> GetCsvTableHeader(string[] columnArray)
        {
            var result = new List<string>();
            foreach (var column in columnArray)
            {
                if (string.IsNullOrWhiteSpace(column))
                {
                    continue;
                }
                result.Add(column);
            }
            return result;
        }

        public static int GetColumnByHeader(List<string> headerList, string header)
        {
            int result = -1;
            for(int index = 0; index < headerList.Count; ++index)
            {
                if (headerList[index].Equals(header))
                {
                    if (result != -1)
                    {
                        throw new TemplateDataException("Duplicate header");
                    }
                    result = index;
                }
            }
            if (result == -1)
            {
                throw new TemplateDataException("Missing header");
            }
            return result;
        }

        public static Dictionary<string, int> GetColumnMappingByHeader(List<string> billHeaderList, List<string> templateHeaderList)
        {
            var result = new Dictionary<string, int>();
            foreach (var templateHeader in templateHeaderList)
            {
                int column = GetColumnByHeader(billHeaderList, templateHeader);
                result.Add(templateHeader, column);
            }
            return result;
        }

        public static Dictionary<string, int> GetColumnMappingByHeaderBypassNoneExist(List<string> billHeaderList, List<string> templateHeaderList)
        {
            var result = new Dictionary<string, int>();
            foreach (var templateHeader in templateHeaderList)
            {
                try
                {
                    int column = GetColumnByHeader(billHeaderList, templateHeader);
                    result.Add(templateHeader, column);
                }
                catch (TemplateDataException)
                {
                    continue;
                }
            }
            return result;
        }

        public static BillTemplateTableEntity GetTemplateEntity(string billType, List<BillTemplateTableEntity> templateList)
        {
            if (string.IsNullOrWhiteSpace(billType))
            {
                throw new TemplateDataException("Not existent bill type");
            }
            if (templateList.Count == 0)
            {
                throw new TemplateDataException("No template defined");
            }
            foreach (var template in templateList)
            {
                if (template.RowKey.Equals(billType, StringComparison.OrdinalIgnoreCase))
                {
                    return template;
                }
            }
            throw new TemplateDataException("Not existent bill type");
        }


        public static int GuessColumnByHeaderNames(List<string> headerList, List<string> candidateNames)
        {
            for (int index = 0; index < headerList.Count; ++index)
            {
                foreach (var name in candidateNames)
                {
                    if (headerList[index].Equals(name))
                    {
                        return index;
                    }
                }
            }
            throw new TemplateDataException("Missing header");
        }

        public static T ParseEnum<T>(string value)
        {
            return (T)Enum.Parse(typeof(T), value, true);
        }

        public static bool OutOfChina(Double lat, Double lng) {
            if ((lng < 72.004) || (lng > 137.8347)) {
                return true;
            }
            if ((lat < 0.8293) || (lat > 55.8271)) {
                return true;
            }
            return false;
        }
  

        public static LatLngModel Bd2Gcj(Double? bdLat, Double? bdLng)
        {
            if (bdLat == null || bdLng == null)
                return new LatLngModel(bdLat, bdLng);
            //if(OutOfChina(bdLat.Value, bdLng.Value))
            //    return new LatLngModel(bdLat,bdLng);

            double x = bdLng.Value - 0.0065;
            double y = bdLat.Value - 0.006;
            double z = Math.Sqrt(x*x + y*y) - 0.00002 * Math.Sin(y * Math.PI);
            double theta = Math.Atan2(y, x) - 0.000003 * Math.Cos(x * Math.PI);
            double gcjLng = z * Math.Cos(theta);
            double gcjLat = z * Math.Sin(theta);
            return new LatLngModel(gcjLat, gcjLng);

        }

        public static LatLngModel Gcj2Bd(Double? gcjLat, Double? gcjLng) {
            
            if (gcjLat == null || gcjLng == null)
                return new LatLngModel(gcjLat, gcjLng);
            //if(OutOfChina(gcjLat.Value, gcjLng.Value))
            //    return new LatLngModel(gcjLat,gcjLng);
  
            double x = gcjLng.Value;
            double y = gcjLat.Value;
            double z = Math.Sqrt(x * x + y * y) + 0.00002 * Math.Sin(y * Math.PI);
            double theta = Math.Atan2(y, x) + 0.000003 * Math.Cos(x * Math.PI);
            var bdLng = z * Math.Cos(theta) + 0.0065;
            var bdLat = z * Math.Sin(theta) + 0.006;
            return new LatLngModel(bdLat, bdLng);
        }


        public static string ReadSetting(string name)
        {
            try
            {
                using (var sqlConn = SQLConnFactory.GetInstance().GetSqlConnection())
                {
                    sqlConn.Open();
                    var queryValue = string.Format("select Value from [CCS_Settings_V1] where Name='{0}'", name);
                    
                    var sqlCmd = new System.Data.SqlClient.SqlCommand(queryValue, sqlConn);
                    var reader = sqlCmd.ExecuteReader();
                    if (reader.Read())
                    {
                        return reader.GetString(0);                       
                    }
                }
            }
            catch (Exception e)
            {
                
            }

            return "";
        }

        public static void WriteSetting(string name, string value)
        {
            try
            {
                using (var sqlConn = SQLConnFactory.GetInstance().GetSqlConnection())
                {
                    sqlConn.Open();
                    var queryValue = string.Format("exec spWriteSetting '{0}', '{1}'", name, value);

                    var sqlCmd = new System.Data.SqlClient.SqlCommand(queryValue, sqlConn);
                    sqlCmd.ExecuteNonQuery();
                    
                }
            }
            catch (Exception e)
            {

            }
        }
    }
}