﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Diagnostics;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Threading;
using System.Threading.Tasks;
using System.Data.Entity;
using System.Data;
using System.Web;
using CCSPrototypeCommon;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Queue;
using Microsoft.WindowsAzure.Storage.Table;
using Microsoft.WindowsAzure.Storage.Table.DataServices;
using Microsoft.WindowsAzure.Storage.RetryPolicies;
using ThreadedWorkerRole;
using Microsoft.Azure;
using Newtonsoft.Json;
using System.Globalization;
using System.Data.Entity.Core.Objects;

namespace HomeWorker
{
    public class TransferBillToDBWorker : WorkerEntryPoint
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }
        private CloudQueue filesQueue;
        private CloudBlobContainer filesBlobContainer;
        private CloudTable _templateTable;

        public override void Run()
        {
            Trace.TraceInformation("TransferBillToDBWorker is running");

            CloudQueueMessage msg = null;
            while (true)
            {
                try
                {
                    msg = this.filesQueue.GetMessage();

                    if (msg != null)
                    {
                        filesQueue.DeleteMessage(msg);
                        ProcessQueueMessage(msg);
                    }
                    else
                    {
                        System.Threading.Thread.Sleep(1000);
                    }
                }
                catch (StorageException e)
                {
                    Trace.TraceError("StorageException in TransferBillToDBWorker: '{0}'", e.Message);
                    logger.logException(e);
                    if (msg != null && msg.DequeueCount > 5)
                    {
                        this.filesQueue.DeleteMessage(msg);
                    }

                    System.Threading.Thread.Sleep(5000);
                }
                catch (Exception e)
                {
                    Trace.TraceError("Exception in TransferBillToDBWorker: '{0}'", e.Message);
                    logger.logException(e);
                }

            }
        }


        private async void ProcessQueueMessage(CloudQueueMessage msg)
        {
            Trace.TraceInformation("Processing queue message '{0}'", msg);

            BillQueueMsg billMsg = await Task.Factory.StartNew(() => JsonConvert.DeserializeObject<BillQueueMsg>(msg.AsString));
            if (string.IsNullOrWhiteSpace(billMsg.BlobName))
            {
                throw new Exception("Null blob name");
            }
            string customerID = Utility.GetCustomerIDFromBlobName(billMsg.BlobName);
            // Insert records in DB
            try
            {
                // Create a retrieve operation that takes a customer entity.
                TableOperation retrieveOperation = TableOperation.Retrieve<BillTemplateTableEntity>(customerID, billMsg.BillType);
                // Execute the retrieve operation.
                TableResult retrievedResult = _templateTable.Execute(retrieveOperation);
                BillTemplateTableEntity billTemplate = null;
                if (retrievedResult != null && retrievedResult.Result != null)
                {
                    billTemplate = retrievedResult.Result as BillTemplateTableEntity;
                }
                else
                {
                    throw new TemplateDataException("No bill template");
                }

                CloudBlockBlob fileBlob = this.filesBlobContainer.GetBlockBlobReference(billMsg.BlobName);
                using (Stream fs = fileBlob.OpenRead())
                using (var db = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                {
                    StreamReader csv = new StreamReader(fs);
                    // Consume header
                    var line = csv.ReadLine();
                    string[] headerList = line.Split(',');
                    var billTemplateMapping = Utility.GetColumnMappingByHeader(new List<string>(headerList), billTemplate.GetColumnList());
                    while ((line = csv.ReadLine()) != null)
                    {
                        Trace.TraceInformation("line {0}", line);
                        string[] columnList = line.Split(',');
                        foreach (string column in columnList)
                        {
                            Trace.TraceInformation("str {0}", column);
                        }
                        var billRecord = new CCS_Bill_V1();

                        if (billTemplate != null)
                        {
                            billRecord.AmountDue = Double.Parse(columnList[billTemplateMapping[billTemplate.AmountDueColumn]], CultureInfo.InvariantCulture);
                            billRecord.BillType = billMsg.BillType;
                            billRecord.CustomerID = Guid.Parse(customerID);
                            billRecord.BillPeriod = DateTime.Parse(columnList[billTemplateMapping[billTemplate.BillPeriodColumn]], CultureInfo.InvariantCulture);
                            if (!string.IsNullOrWhiteSpace(billTemplate.AccountNumberColumn))
                            {
                                billRecord.AccountNumber = columnList[billTemplateMapping[billTemplate.AccountNumberColumn]];
                            }
                            if (!string.IsNullOrWhiteSpace(billTemplate.UserNumberColumn))
                            {
                                billRecord.UserNumber = columnList[billTemplateMapping[billTemplate.UserNumberColumn]];
                            }
                            var userRecord = new CCS_User_V1();
                            userRecord.FirstName = columnList[billTemplateMapping[billTemplate.UserNameColumn]];
                            userRecord.Cellphone = columnList[billTemplateMapping[billTemplate.CellphoneColumn]];
                            userRecord.Address = columnList[billTemplateMapping[billTemplate.UserAddressColumn]];
                            userRecord.ModifiedDate = DateTime.UtcNow;
                            try
                            {
                                var IDNumberHeaders = new List<string>(new string[]
                                {
                                    "身份证", "身份证号"
                                });
                                var IDNumberColumn = Utility.GuessColumnByHeaderNames(new List<string>(headerList), IDNumberHeaders);
                                userRecord.IDNumber = columnList[IDNumberColumn];
                            }
                            catch (TemplateDataException e)
                            {
                                // This is expected in most cases
                            }
                            billRecord.UserID = await GetUserGUID(userRecord, true);

                            var result = (from bill in db.BillTable.AsNoTracking().Where(b => b.BillPeriod.Year == billRecord.BillPeriod.Year &&
                                b.BillPeriod.Month == billRecord.BillPeriod.Month && b.CustomerID == billRecord.CustomerID &&
                                (string.IsNullOrEmpty(billRecord.AccountNumber) || b.AccountNumber == billRecord.AccountNumber) &&
                                (string.IsNullOrEmpty(billRecord.UserNumber) || b.UserNumber == billRecord.UserNumber) &&
                                b.ModifiedDate == null && b.BillType == billRecord.BillType)
                                      select new
                                      {
                                          Bill = bill
                                      });
                            var record = result.ToList();
                            if (record == null || record.Count == 0)
                            {
                                billRecord.ModifiedDate = null;
                                db.BillTable.Add(billRecord);
                            }
                            else
                            {
                                var billGuid = await MergeBillRecord(billRecord, record.FirstOrDefault().Bill);
                                await UpsertBillInfoRecord(billGuid, headerList, columnList);
                                continue;
                            }
                            for (var index = 0; index < headerList.Length; ++index)
                            {
                                if (string.IsNullOrWhiteSpace(headerList[index]))
                                {
                                    continue;
                                }
                                var billInfoRecord = new CCS_BillInfo_V1();
                                billInfoRecord.Name = headerList[index];
                                billInfoRecord.Value = columnList[index];
                                db.BillInfoTable.Add(billInfoRecord);

                                string headerColumn = headerList[index];
                                try
                                {
                                    if (!db.BillTemplateTable.Any(obj => obj.BillType == billMsg.BillType && obj.Column == headerColumn))
                                    {
                                        var billTemplateRow = new CCS_BillTemplate_V1();
                                        billTemplateRow.BillType = billMsg.BillType;
                                        billTemplateRow.CustomerID = billRecord.CustomerID;
                                        billTemplateRow.Column = headerColumn;
                                        billTemplateRow.ColumnType = ParseValue(columnList[index]);
                                        db.BillTemplateTable.Add(billTemplateRow);
                                    }
                                }
                                catch (Exception e)
                                {
                                    // Ignore exceptions
                                    logger.logException(e);
                                }
                            }
                        }
                        else
                        {
                            throw new TemplateDataException("No bill template");
                        }
                        await db.SaveChangesAsync();
                    }
                }
            }
            catch (Exception e)
            {
                Trace.TraceError(String.Format("Exception in ProcessQueueMessage: '{0}'", e.Message));
                logger.logException(e);
            }

        }

        private async Task UpsertBillInfoRecord(Guid billGuid, string[] headerList, string[] columnList)
        {
            try
            {
                using (var dbInsert = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                using (var dbUpdate = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                {
                    var result = (from billInfo in dbUpdate.BillInfoTable.AsNoTracking().Where(s => (s.BillID == billGuid))
                                  select new
                                  {
                                      BillInfo = billInfo
                                  });
                    var recordList = result.ToList();
                    var oldInfo = new Dictionary<string, string>();
                    foreach (var record in recordList)
                    {
                        oldInfo.Add(record.BillInfo.Name, record.BillInfo.Value);
                        record.BillInfo.Status = DBItemState.Invalid;
                        dbUpdate.BillInfoTable.Attach(record.BillInfo);
                        dbUpdate.Entry(record.BillInfo).State = System.Data.Entity.EntityState.Modified;
                    }
                    var changed = false;
                    for (var index = 0; index < headerList.Length; ++index)
                    {
                        if (string.IsNullOrWhiteSpace(headerList[index]) || string.IsNullOrWhiteSpace(columnList[index]))
                        {
                            continue;
                        }
                        if (!oldInfo.ContainsKey(headerList[index]) ||
                            (oldInfo.ContainsKey(headerList[index]) && string.Compare(oldInfo[headerList[index]], columnList[index], true) != 0))
                        {
                            changed = true;
                        }
                        var billInfo = new CCS_BillInfo_V1();
                        billInfo.BillID = billGuid;
                        billInfo.Name = headerList[index];
                        billInfo.Value = columnList[index];
                        dbInsert.BillInfoTable.Add(billInfo);
                    }
                    if (changed)
                    {
                        await dbInsert.SaveChangesAsync();
                        await dbUpdate.SaveChangesAsync();
                    }
                }
            }
            catch (Exception e)
            {
                Trace.TraceError(String.Format("Exception: '{0}'", e.Message));
                logger.logException(e);
                throw;
            }
        }

        private async Task<Guid> GetUserGUID(CCS_User_V1 userRecord, bool addIfNotExist)
        {
            try
            {
                using (var dbInsert = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                using (var dbUpdate = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                {
                    var result = (from user in dbUpdate.UserTable.AsNoTracking().Where(s => (
                        (s.FirstName == null || s.FirstName.Trim() == string.Empty || userRecord.FirstName == null || userRecord.FirstName.Trim() == string.Empty || s.FirstName == userRecord.FirstName) &&
                        (s.LastName == null || s.LastName.Trim() == string.Empty || userRecord.LastName == null || userRecord.LastName.Trim() == string.Empty || s.FirstName == userRecord.LastName) &&
                        (s.Cellphone == null || s.Cellphone.Trim() == string.Empty || userRecord.Cellphone == null || userRecord.Cellphone.Trim() == string.Empty || s.Cellphone == userRecord.Cellphone) &&
                        (s.Email == null || s.Email.Trim() == string.Empty || userRecord.Email == null || userRecord.Email.Trim() == string.Empty || s.Email == userRecord.Email) &&
                        (s.Address == null || s.Address.Trim() == string.Empty || userRecord.Address == null || userRecord.Address.Trim() == string.Empty || s.Address == userRecord.Address)))
                            select new
                            {
                                User = user
                            });
                    var record = result.ToList();
                    if (record != null && record.Count > 1)
                    {
                        if (!string.IsNullOrWhiteSpace(userRecord.IDNumber))
                        {
                            record = result.Where(s => (s.User.IDNumber == userRecord.IDNumber)).ToList();
                        }
                        if (record.Count > 1)
                        {
                            foreach (var user in record)
                            {
                                var error = string.Format("Duplicate user info at userid: {0}", user.User.UserID);
                                Trace.TraceError(error);
                                logger.logException(new Exception(error));
                            }
                        }
                    }
                    if (record == null || record.Count == 0)
                    {
                        if (addIfNotExist)
                        {
                            dbInsert.UserTable.Add(userRecord);
                            await dbInsert.SaveChangesAsync();
                            return userRecord.UserID;
                        }
                        else
                        {
                            var error = string.Format("Failed to find user name: {0}; cellphone: {1}; address: {2};",
                                userRecord.FirstName, userRecord.Cellphone, userRecord.Address);
                            throw new Exception(error);
                        }
                    }
                    else
                    {
                        if (string.IsNullOrWhiteSpace(userRecord.Address))
                        {
                            userRecord.Address = record.FirstOrDefault().User.Address;
                        }
                        if (string.IsNullOrWhiteSpace(userRecord.Cellphone))
                        {
                            userRecord.Cellphone = record.FirstOrDefault().User.Cellphone;
                        }
                        if (string.IsNullOrWhiteSpace(userRecord.FirstName))
                        {
                            userRecord.FirstName = record.FirstOrDefault().User.FirstName;
                        }
                        if (string.IsNullOrWhiteSpace(userRecord.LastName))
                        {
                            userRecord.LastName = record.FirstOrDefault().User.LastName;
                        }
                        if (string.IsNullOrWhiteSpace(userRecord.Email))
                        {
                            userRecord.Email = record.FirstOrDefault().User.Email;
                        }
                        if (string.IsNullOrWhiteSpace(userRecord.IDNumber))
                        {
                            userRecord.IDNumber = record.FirstOrDefault().User.IDNumber;
                        }
                        if (string.IsNullOrWhiteSpace(userRecord.iOSDeviceToken))
                        {
                            userRecord.iOSDeviceToken = record.FirstOrDefault().User.iOSDeviceToken;
                        }
                        if (string.IsNullOrWhiteSpace(userRecord.SecondPhone))
                        {
                            userRecord.SecondPhone = record.FirstOrDefault().User.SecondPhone;
                        }
                        userRecord.ModifiedDate = DateTime.UtcNow;
                        userRecord.UserID = record.FirstOrDefault().User.UserID;
                        dbUpdate.UserTable.Attach(userRecord);
                        dbUpdate.Entry(userRecord).State = System.Data.Entity.EntityState.Modified;
                        await dbUpdate.SaveChangesAsync();
                    }
                    return record.FirstOrDefault().User.UserID;
                }
            }
            catch (Exception e)
            {
                Trace.TraceError(String.Format("Exception: '{0}'", e.Message));
                logger.logException(e);
                throw;
            }
        }

        private async Task<Guid> MergeBillRecord(CCS_Bill_V1 newRecord, CCS_Bill_V1 oldRecord)
        {
            if (newRecord.BillType != oldRecord.BillType ||
                newRecord.UserID != oldRecord.UserID ||
                newRecord.AmountDue != oldRecord.AmountDue)
            {
                try
                {
                    using (var dbInsert = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                    using (var dbUpdate = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                    {
                        dbInsert.BillTable.Add(newRecord);
                        await dbInsert.SaveChangesAsync();
                        oldRecord.ModifiedDate = DateTime.UtcNow;
                        dbUpdate.BillTable.Attach(oldRecord);
                        dbUpdate.Entry(oldRecord).State = System.Data.Entity.EntityState.Modified;
                        await dbUpdate.SaveChangesAsync();
                        return newRecord.BillID;
                    }
                }
                catch (Exception e)
                {
                    Trace.TraceError(String.Format("Exception: '{0}'", e.Message));
                    logger.logException(e);
                    throw;
                }
            }
            return oldRecord.BillID;
        }

        private string ParseValue(string value)
        {
            string result;
            Int32 intValue;
            Int64 bigIntValue;
            Double doubleValue;
            DateTime dateValue;

            if (Int32.TryParse(value, out intValue) || Int64.TryParse(value, out bigIntValue))
            {
                result = "Integer";
            }
            else if (Double.TryParse(value, out doubleValue))
            {
                result = "Double";
            }
            else if (DateTime.TryParse(value, out dateValue))
            {
                result = "DateTime";
            }
            else
            {
                result = "String";
            }

            return result;
        }


        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections
            ServicePointManager.DefaultConnectionLimit = 12;

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.


            // Open storage account
            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(ConfigurationMgr.GetSetting("StorageConnectionString"));

            // Create blob container
            CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
            filesBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.BillStorageName);
            if (filesBlobContainer.CreateIfNotExists())
            {
                filesBlobContainer.SetPermissions(
                    new BlobContainerPermissions
                    {
                        PublicAccess = BlobContainerPublicAccessType.Blob
                    });
            }

            // Create quere storage
            CloudQueueClient queueClient = storageAccount.CreateCloudQueueClient();
            filesQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.BillStorageName);
            filesQueue.CreateIfNotExists();

            // Get table context object
            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();
            tableClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            _templateTable = tableClient.GetTableReference(Constants.BillCsvTemplateTableName);
            _templateTable.CreateIfNotExists();

            Trace.TraceInformation("Storage initialized");

            return base.OnStart();
        }

    }
}
